# CEA_WSN_JAXWS_Setup.py - CEA WS-Notification setup script
# 
# Licensed Materials  Property of IBM
#  Copyright IBM Corp. 2009, 2009, All Rights Reserved
# US Government Users Restricted Rights  Use, duplication or disclosure restricted by GSA ASP
# Schedule Contract with IBM Corp.
# 
# Note:
#    - This script assumes that you have administrative security disabled.
# 
# Execute the script by typing:
# 
#   wsadmin -f CEA_WSN_JAXWS_Setup.py

# Reset the configuration environment:
AdminConfig.reset( )

scriptName = "CEA_WSN_JAXWS_Setup.py"

# Setup the arguments:
busName = "CEABus"
serviceName = "CEAService"
servicePointName = "CEAServicePoint"

print "The following input arguments have been specified:"
print "SI Bus            : " + busName
print "WSN service       : " + serviceName
print "WSN service point : " + servicePointName
print ""

# Get server and node objects and verify we are in a single server environment:

# Get line separator:
lineSeparator = java.lang.System.getProperty('line.separator')

# Get the names of the node and server objects, and at the same time verify we are running in a single node and server environment:
nodeObjectsArray = AdminConfig.list("Node").split(lineSeparator)

# Ensure that we are in a single node environment:
if (len(nodeObjectsArray) != 1):
  print "You have more than one node defined - please rerun this script in a single node environment."
  sys.exit()
#endIf 

nodeName = AdminConfig.showAttribute(nodeObjectsArray[0], "name")

serverObjectsArray = AdminConfig.list("Server").split(lineSeparator)

# Ensure that we are in a single server environment:
if (len(serverObjectsArray) != 1):
  print "You have more than one server defined - please rerun this script in a single server environment."
  sys.exit()
#endIf 

serverName = AdminConfig.showAttribute(serverObjectsArray[0], "name")

print "Inferred names:"
print "Node name         : " + nodeName
print "Server name       : " + serverName

print ""
print "### STEP 1 of 6. CREATE AN SI BUS ##########################################################"
print ""

# Make sure the bus we are trying to create doesn't already exist:
found = 0
SIBusList = AdminConfig.list("SIBus").splitlines()
for SIBusObject in SIBusList:
        print "SIBusObject: " + SIBusObject
        if (cmp(AdminConfig.showAttribute(SIBusObject, "name" ), busName) == 0):
                found = 1
        #endIf 
#endFor 

# Exit the script if bus already exists:
if (found == 1):
        print scriptName + " FAILED - An SI Bus called " + busName + " already exists! Script aborted."
        sys.exit()
#endIf 

# Since the bus does not already exist we can create it:
description = "Bus for CEA WS-Notifications, created by " + scriptName
args = "-bus " + busName + " -description '" + description + "'"

# Ensure we deal with any error from the create command:
print "About to create SI Bus using args: " + args
try:
        _excp_ = 0
        csb = AdminTask.createSIBus(args)
        result = csb
        _excp_ = 0 #reset (in case of nested exceptions)
except:
        _type_, _value_, _tbck_ = sys.exc_info()
        result = str(_value_)
        _excp_ = 1
#endTry 
if (_excp_ ):
        print scriptName + " FAILED to create an SI Bus! Error: " + result
        sys.exit()
#endIf 

print "Successfully created an SI Bus called: " + busName

print ""
print "### STEP 2 of 6. ADD BUS MEMBER TO THE SI BUS ##############################################"
print ""

args = "-bus " + busName + " -node " + nodeName + " -server " + serverName
print "About to add SI Bus member using args: " + args
try:
        _excp_ = 0
        csb = AdminTask.addSIBusMember(args)
        result = csb
        _excp_ = 0 #reset (in case of nested exceptions)
except:
        _type_, _value_, _tbck_ = sys.exc_info()
        result = str(_value_)
        _excp_ = 1
#endTry 
if (_excp_ ):
        print scriptName + " FAILED to add SI Bus member! Error: " + result
        sys.exit()
#endIf

print "Successfully added bus member to SI Bus: " + busName

print ""
print "### STEP 3 of 6. CREATE WS-NOTIFICATION SERVICE ############################################"
print ""

description = "WSN service for CEA WS-Notifications, created by " + scriptName
args = "-name " + serviceName + " -description '" + description + "' -bus " + busName + " -type V7.0"

print "About to create a WSN service using args: " + args
try:
        _excp_ = 0
        inService = AdminTask.createWSNService(args)
        result = inService
        _excp_ = 0 #reset (in case of nested exceptions)
except:
        _type_, _value_, _tbck_ = sys.exc_info()
        result = str(_value_)
        _excp_ = 1
#endTry 
if (_excp_ ):
        print scriptName + " FAILED to create a WSN service! Error: " + result
        sys.exit()
#endIf

print "Successfully created WSN service: " + serviceName

print ""
print "### STEP 4 of 6. CREATE WS-NOTIFICATION SERVICE POINT ######################################"
print ""

# We need the target WSN service object before we can create a WSN service point on it...
args = "-name " + serviceName + " -bus " + busName
print "About to locate the target WSN service object using args: " + args
try:
        _excp_ = 0
        target = AdminTask.listWSNServices(args)
        result = target
        _excp_ = 0 #reset (in case of nested exceptions)
except:
        _type_, _value_, _tbck_ = sys.exc_info()
        result = str(_value_)
        _excp_ = 1
#endTry 
if (_excp_ ):
        print scriptName + " FAILED to locate the target WSN service object! Error: " + result
        sys.exit()
#endIf 

print "Found target WSN service: " + target

description = "WSN service point for CEA WS-Notifications, created by " + scriptName
args = "-name " + servicePointName + " -server " + serverName + " -node " + nodeName + " -description '" + description + "' "

#--- Create the WSN Service Point

print "About to create a WSN service point using args: " + args
try:
        _excp_ = 0
        servicePoint = AdminTask.createWSNServicePoint(target, args)
        result = servicePoint
        _excp_ = 0 #reset (in case of nested exceptions)
except:
        _type_, _value_, _tbck_ = sys.exc_info()
        result = str(_value_)
        _excp_ = 1
#endTry 
if (_excp_ ):
        print scriptName + " FAILED to create a WSN service point! Error: " + result
        sys.exit()
#endIf 

print "Successfully created WSN service point: " + servicePointName

print ""
print "### STEP 5 of 6. SAVE CHANGES TO THE CONFIGURATION #########################################"
print ""

print "About to save changes to the configuration..."
AdminConfig.save( )
print "Configuration changes have been saved."

print ""
print "### STEP 6 of 6. START THE SERVICE POINT APPLICATION #######################################"
print ""

# Note: The service point application is not installed until the configuration changes have been
# saved. So only now can we start it...
 
appName = "WSN_" + serviceName + "_" + servicePointName
print "About to start the service point application: " + appName

# Identify the application manager MBean for the server where the application resides:
appManager = AdminControl.completeObjectName("type=ApplicationManager,node=" + nodeName + ",process=" + serverName + ",*")
try:
        _excp_ = 0
        result = AdminControl.invoke(appManager, "startApplication", appName)
        _excp_ = 0 #reset (in case of nested exceptions)
except:
        _type_, _value_, _tbck_ = sys.exc_info()
        result = str(_value_)
        _excp_ = 1
#endTry 
if (_excp_ ):
        print scriptName + " FAILED to start the service point application! Error: " + result
        sys.exit()
#endIf 

print "Successfully started the WSN service point application: " + appName
print ""

# Output success string
print scriptName + " has completed successfully."
