import  sys
import  time

def _splitlines(s):
    rv = [s]
    if '\r' in s:
        rv = s.split('\r\n')
    elif '\n' in s:
        rv = s.split('\n')
    if rv[-1] == '':
        rv = rv[:-1]
    return rv

def stringListToList(stringList):
    """Nuisance helper method converts a string which looks like a list into a list of strings.

    Expects string input of the form [(cells/ding/...0669) (cells/ding/...6350) (cells/ding/...6291)]
    Returns string output of the form  ['(cells/ding/...0669)', '(cells/ding/...6350)', '(cells/ding/...6291)']
    Note: string.split does not work because of the square brackets."""
    m = "stringListToList:"
    sop(m,"Entry. stringList=%s" % ( stringList ))

    # Bozo check.
    if not (stringList.startswith( '[' ) and stringList.endswith( ']' )):
        raise m + " ERROR: stringList does not start and end with square brackets. stringList=%s" % ( m, stringList, )

    # Strip off the leading and trailing square brackets.
    stringList = stringList[1:(len(stringList) - 1)]

    # Get rid of the leading and trailing square brackets.
    listList = stringList.split()

    sop(m,"Exit. listList=%s" % (listList))
    return listList

def getSopTimestamp():
    """Returns the current system timestamp in a nice format."""
    return time.strftime('[%Y-%m%d-6/29/09M-%S00]')

def sop(methodname,message):
    """Prints the specified method name and message with a nicely formatted timestamp.
    (sop is an acronym for System.out.println() in java)"""
    timestamp = getSopTimestamp()
    print "%s %s %s" % (timestamp, methodname, message)

def disableApp():
    # Disable commsvc app by default

    deployments = AdminConfig.getid('/Deployment:commsvc/')
    if (len(deployments) > 0) :
        deploymentObj = AdminConfig.showAttribute(deployments, 'deployedObject')
        targetMappings = stringListToList(AdminConfig.showAttribute(deploymentObj, 'targetMappings'))
   
        for targetMapping in targetMappings:
            sop (m,"Disabling commsvc on target mapping %s" % ( repr(targetMapping) )) 
            AdminConfig.modify(targetMapping,[['enable', 'false']])

m = "installCOMMSVC.py:"
#print "===================================="
sop(m,"Entry")
ear = sys.argv[0]
cells = _splitlines(AdminConfig.list('Cell'))

for cell in cells :
     nodenames = []
     servernames = []
     clusters = []
     memberNames = []
     memberNodes = []
     hasDmgr = 'false'
     hasNodeAgent = 'false'
     cname = AdminConfig.showAttribute(cell, 'name')
     nodes = _splitlines(AdminConfig.list('Node', cell))


     cluster_ids = _splitlines(AdminConfig.list( 'ServerCluster' ))
     for cluster_id in cluster_ids :
          clusters.append(AdminConfig.showAttribute(cluster_id, 'name'))
          memberlist = _splitlines(AdminConfig.list('ClusterMember', cluster_id))
          for member in memberlist :
               memberNames.append(AdminConfig.showAttribute(member, 'memberName'))
               memberNodes.append(AdminConfig.showAttribute(member, 'nodeName'))
     
     if (len(cluster_ids) > 0) :
          sop(m, "compiled clusters=%s memberNames=%s memberNodes=%s" % (repr(clusters), repr(memberNames), repr(memberNodes)))
     else :
          sop(m, "no clusters found in %s" % (cell))
     
     sop(m,"Examining all nodes and servers in cell %s." % ( cname ))
     for node in nodes :
          nname = AdminConfig.showAttribute(node, 'name')
          servs = _splitlines(AdminConfig.list( 'ServerEntry', node ))
          
          for server in servs :
               sname = AdminConfig.showAttribute( server, "serverName" )
               stype = AdminConfig.showAttribute( server, "serverType" )
               sop(m,"Handling cell=%s node=%s server=%s stype=%s" % (cname,nname,sname,stype))
               
               if stype == 'DEPLOYMENT_MANAGER' :
                    hasDmgr = 'true'
                    
               if stype == 'NODE_AGENT' :
                    hasNodeAgent = 'true'
                    
               if stype == 'APPLICATION_SERVER' :
                    found = 'false'
                    
                    for i in range (0, len(memberNames)) :
                         if sname == memberNames[i] and nname == memberNodes[i] :
                              found = 'true'
                              
                    if found == 'false' :
                         nodenames.append(nname)
                         servernames.append(sname)
                         
          
     sop(m,"Summary: hasDmgr=%s hasNodeAgent=%s cellname=%s nodenames=%s servernames=%s" % ( hasDmgr, hasNodeAgent, cname, repr(nodenames), repr(servernames)))
     
     
     if(hasDmgr == 'true') :
         # Install the CEA system app on all appservers and clusters known by the dmgr.
         
         sop(m,"Handling dmgr. len(nodenames)=%i" % ( len(nodenames) ))
         
         if (0 < len(nodenames)):
             
             # Define the module mapping
             webcon = 'WebSphere:cell=' + cname + ',node=' + nodenames[0] + ',server=' + servernames[0]
             for i in range (1 ,len(nodenames)) :
                 webcon = webcon + '+' + 'WebSphere:cell=' + cname + ',node=' + nodenames[i] + ',server=' + servernames[i]
             
             for i in range (0 ,len(clusters)) :
                 webcon = webcon + '+' + 'WebSphere:cell=' + cname + ',cluster=' + clusters[i]
             
             # Define arguments for the install command using a python list of strings.
             arglist = ['-appname', 'commsvc', 
                        '-cell', cname,
                        '-node', nodenames[0],
                        '-server', servernames[0],
                        '-skipPreparation', 
                        '-zeroEarCopy', 
                        '-installed.ear.destination', ear[:-11]]
             sop(m,"arglist=%s" % ( repr(arglist) ))
             
             sop(m,"Calling AdminApp with arglist.")
             AdminApp.install(ear, arglist)
             
             # Or define arguments for the install command using one long string.
             argstring = '[-MapModulesToServers [[ .* .* %s ]]]' % ( webcon )
             sop(m,"arglist=%s" % ( repr(argstring) ))
             
             sop(m,"Calling AdminApp with argstring.")
             AdminApp.edit('commsvc', argstring)
             disableApp()
         elif (len(clusters) > 0) :
             # Define the module mapping
             webcon = 'WebSphere:cell=' + cname + ',cluster=' + clusters[0]
             
             for i in range (1 ,len(clusters)) :
                 webcon = webcon + '+' + 'WebSphere:cell=' + cname + ',cluster=' + clusters[i]
             
             # Define arguments for the install command using a python list of strings.
             arglist = ['-appname', 'commsvc', 
                        '-cell', cname,
                        '-cluster', clusters[0], 
                        '-skipPreparation', 
                        '-zeroEarCopy', 
                        '-installed.ear.destination', ear[:-11]]
             sop(m,"arglist=%s" % ( repr(arglist) ))
             
             sop(m,"Calling AdminApp with arglist.")
             AdminApp.install(ear, arglist)
             
             # Or define arguments for the install command using one long string.
             argstring = '[-MapModulesToServers [[ .* .* %s ]]]' % ( webcon )
             sop(m,"arglist=%s" % ( repr(argstring) ))
             
             sop(m,"Calling AdminApp with argstring.")
             AdminApp.edit('commsvc', argstring)
             disableApp()
         else:
             sop(m,"No appservers or clusters on which to install.")
             
     elif (hasDmgr == "false" and hasNodeAgent == "true"):   
         # Do not install the CEA system app on nodes which have already been federated.
         # For these nodes, the CEA system app is installed when the Dmgr profile is augmented (previous paragraph).
         sop(m,"Skipping previously-federated node(s).")
         
     else:
         # Install the CEA system app on standalone non-federated nodes.
         sop(m,"Handling non-federated node (no dmgr and no nodeagent).")
         
         if (0 == len(nodenames)):
             # Skip a profile with no appservers, such as a custom profile.
             sop(m,"Continuing. There are no appservers on which to install the CEA system app.")
         elif (1 == len(nodenames)):
             # Handle a standalone profile with one appserver.
             arglist = ['-cell', cname, 
                        '-node', nodenames[0], 
                        '-server', servernames[0], 
                        '-appname', 'commsvc', 
                        '-skipPreparation', 
                        '-zeroEarCopy', 
                        '-installed.ear.destination', ear[:-11]]
             sop(m,"Calling AdminApp.install with arglist=%s" % ( repr(arglist) ))
             AdminApp.install(ear, arglist)
             disableApp()
         else:
             msg = "ERROR: non-dmgr non-nodeagent configuration has more than one appserver. numServers=%i" % ( len(nodename) )
             sop(m,msg)
             raise msg     
     sop(m,"Saving")
     AdminConfig.save()


sop(m,"Exit")
#print "===================================="
