import  sys
import  time

def _splitlines(s):
    rv = [s]
    if '\r' in s:
        rv = s.split('\r\n')
    elif '\n' in s:
        rv = s.split('\n')
    if rv[-1] == '':
        rv = rv[:-1]
    return rv

def stringListToList(stringList):
    """Nuisance helper method converts a string which looks like a list into a list of strings.

    Expects string input of the form [(cells/ding/...0669) (cells/ding/...6350) (cells/ding/...6291)]
    Returns string output of the form  ['(cells/ding/...0669)', '(cells/ding/...6350)', '(cells/ding/...6291)']
    Note: string.split does not work because of the square brackets."""
    m = "stringListToList:"
    sop(m,"Entry. stringList=%s" % ( stringList ))

    # Bozo check.
    if not (stringList.startswith( '[' ) and stringList.endswith( ']' )):
        raise m + " ERROR: stringList does not start and end with square brackets. stringList=%s" % ( m, stringList, )

    # Strip off the leading and trailing square brackets.
    stringList = stringList[1:(len(stringList) - 1)]

    # Get rid of the leading and trailing square brackets.
    listList = stringList.split()

    sop(m,"Exit. listList=%s" % (listList))
    return listList

def getSopTimestamp():
    """Returns the current system timestamp in a nice format."""
    return time.strftime('[%Y-%m%d-6/29/09M-%S00]')

def sop(methodname,message):
    """Prints the specified method name and message with a nicely formatted timestamp.
    (sop is an acronym for System.out.println() in java)"""
    timestamp = getSopTimestamp()
    print "%s %s %s" % (timestamp, methodname, message)

def disableApp():
    # Disable commsvc app by default

    deployments = AdminConfig.getid('/Deployment:commsvc/')
    if (len(deployments) > 0) :
        deploymentObj = AdminConfig.showAttribute(deployments, 'deployedObject')
        targetMappings = stringListToList(AdminConfig.showAttribute(deploymentObj, 'targetMappings'))
   
        for targetMapping in targetMappings:
            sop (m,"Disabling commsvc on target mapping %s" % ( repr(targetMapping) )) 
            AdminConfig.modify(targetMapping,[['enable', 'false']])

def reenableApp():
    # Re-enable commsvc app on servers and clusters it was previously enabled on
    deployments = AdminConfig.getid('/Deployment:commsvc/')
    if (len(deployments) > 0) :
        deploymentObj = AdminConfig.showAttribute(deployments, 'deployedObject')
        targetMappings = stringListToList(AdminConfig.showAttribute(deploymentObj, 'targetMappings'))
   
        # for each target mapping in the deployment
        for targetMapping in targetMappings:
            target = AdminConfig.showAttribute(targetMapping, 'target') 
             
            # if it is a server targeted, check the server name and node name and enable mapping if match is found
            if(target.find('ServerTarget') != -1) :
                 nodeName = AdminConfig.showAttribute(target,'nodeName')
                 serverName = AdminConfig.showAttribute(target,'name')
                 
                 for i in range (0 ,len(enabledNodes)) :
                     if( enabledNodes[i] == nodeName and enabledServers[i] == serverName ):
                          AdminConfig.modify(targetMapping,[['enable', 'true']])
            
            #if its a cluster targeted, check the cluster name and enable mapping if match is found
            if(target.find('ClusteredTarget') != -1) :
                 clusterName = AdminConfig.showAttribute(target,'name')

                 for i in range (0 ,len(enabledClusters)) :
                     if( enabledClusters[i] == clusterName ):
                          AdminConfig.modify(targetMapping,[['enable', 'true']])
                 
m = "addDeploymentOfCommsvc.py:"
#print "===================================="
sop(m,"Entry")
ear = sys.argv[0] + '/feature_packs/cea/systemApps/commsvc.ear'
cells = _splitlines(AdminConfig.list('Cell'))


for cell in cells :
  cname = AdminConfig.showAttribute(cell, 'name')
     
  if(cname == sys.argv[1]) :
     enabledServers = []
     enabledNodes = []
     enabledClusters = []
     nodenames = []
     servernames = []
     clusternames = []
     
     deployments = AdminConfig.getid('/Deployment:commsvc/')
     
     # If the system app is installed on the Deployment manager
     if (len(deployments) > 0) :
         deploymentObj = AdminConfig.showAttribute(deployments, 'deployedObject')
         targetMappings = stringListToList(AdminConfig.showAttribute(deploymentObj, 'targetMappings'))
         
         # for each target mapping in the deployment
         for targetMapping in targetMappings:
             target = AdminConfig.showAttribute(targetMapping, 'target') 
             
             # if it is a server targeted, save the server name and node name
             if(target.find('ServerTarget') != -1) :
                 # save the target mapping if it is enabled
                 if('true' == AdminConfig.showAttribute(targetMapping,'enable')) :
                     enabledNodes.append(AdminConfig.showAttribute(target,'nodeName'))
                     enabledServers.append(AdminConfig.showAttribute(target,'name'))
                 nodenames.append(AdminConfig.showAttribute(target,'nodeName'))
                 servernames.append(AdminConfig.showAttribute(target,'name'))

             #if its a cluster targeted, save the cluster name
             if(target.find('ClusteredTarget') != -1) :
                 # save the target mapping if it is enabled
                 if('true' == AdminConfig.showAttribute(targetMapping,'enable')) :
                     enabledClusters.append(AdminConfig.showAttribute(target,'name'))
                 clusternames.append(AdminConfig.showAttribute(target,'name'))
              
         #if you are augmenting a cluster, append the cluster name
         if(len(sys.argv) == 3) :
             clusternames.append(sys.argv[2])

         #if you are augmenting a federated server, append the server name and node name
         if(len(sys.argv) == 4) :
             nodenames.append(sys.argv[2])
             servernames.append(sys.argv[3])
         
         sop(m,"Summary: cellname=%s nodenames=%s servernames=%s clusternames=%s" % ( cname, repr(nodenames), repr(servernames), repr(clusternames)))
         
         # if there are servers in the deployment mapping
         if (0 < len(nodenames)):
             # Define the module mapping
             webcon = 'WebSphere:cell=' + cname + ',node=' + nodenames[0] + ',server=' + servernames[0]
             for i in range (1 ,len(nodenames)) :
                 webcon = webcon + '+' + 'WebSphere:cell=' + cname + ',node=' + nodenames[i] + ',server=' + servernames[i]
         
             # if there are also clusters in the mapping    
             for i in range (0 ,len(clusternames)) :
                 webcon = webcon + '+' + 'WebSphere:cell=' + cname + ',cluster=' + clusternames[i]
         
             # Or define arguments for the install command using one long string.
             argstring = '[-MapModulesToServers [[ .* .* %s ]]]' % ( webcon )
             sop(m,"arglist=%s" % ( repr(argstring) ))
             
             sop(m,"Calling AdminApp with argstring.")

             # update target mappings to include added target
             AdminApp.edit('commsvc', argstring)

             # disable all target mappings
             disableApp()
         
             # re-enable the mappings that were enabled before
             reenableApp()
     
         # if there are ONLY clusters in the environment
         elif( 0 < len(clusternames)):
             webcon = 'WebSphere:cell=' + cname + ',cluster=' + clusternames[0]
             for i in range (1 ,len(clusternames)) :
                 webcon = webcon + '+' + 'WebSphere:cell=' + cname + ',cluster=' + clusternames[i]        
             
             # Or define arguments for the install command using one long string.
             argstring = '[-MapModulesToServers [[ .* .* %s ]]]' % ( webcon )
             sop(m,"arglist=%s" % ( repr(argstring) ))
                 
             sop(m,"Calling AdminApp with argstring.")

             # update target mappings to include added target
             AdminApp.edit('commsvc', argstring)

             # disable all target mappings
             disableApp()
         
             # re-enable the mappings that were enabled before
             reenableApp()
     
     # If the system app is not yet installed on the deployment manager
     else:
         # Define arguments for the install on a cluster using a python list of strings.
         if(len(sys.argv) == 3) :
             arglist = ['-appname', 'commsvc', 
                        '-cell', sys.argv[1],
                        '-cluster', sys.argv[2],
                        '-skipPreparation', 
                        '-zeroEarCopy', 
                        '-installed.ear.destination', ear[:-11]]

             sop(m,"arglist=%s" % ( repr(arglist) ))
             
             sop(m,"Calling AdminApp with arglist.")
             AdminApp.install(ear, arglist)
             disableApp()

         # Define arguments for the install on a federated app server using a python list of strings.
         if(len(sys.argv) == 4) :
             arglist = ['-appname', 'commsvc', 
                        '-cell', sys.argv[1],
                        '-node', sys.argv[2],
                        '-server', sys.argv[3],
                        '-skipPreparation', 
                        '-zeroEarCopy', 
                        '-installed.ear.destination', ear[:-11]]
             sop(m,"arglist=%s" % ( repr(arglist) ))
             
             sop(m,"Calling AdminApp with arglist.")
             AdminApp.install(ear, arglist)
             disableApp()
             
sop(m,"Saving")
AdminConfig.save()

sop(m,"Exit")