# All Rights Reserved - Licensed Materials - Property of IBM
# 5724-J08, 5724-I63, 5724-H88, 5724-H89, 5655-N02, 5733-W70 COPYRIGHT International Business Machines Corp., 2009, 2010
# US Government Users Restricted Rights - Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.

import sys
import time

## Configure cell-level CEA settings ##
## configureCEASettingsForCell("/commsvc.rest", "default_host")

def configureCEASettingsForCell(contextRoot, virtualHost):
   AdminApp.edit("commsvc", "[ -CtxRootForWebMod [[ commsvc commsvc.rest.war,WEB-INF/web.xml " + contextRoot + " ]]]" )
   AdminApp.edit("commsvc", "[ -MapWebModToVH [[ commsvc commsvc.rest.war,WEB-INF/web.xml " + virtualHost + " ]]]" )
#endDef


## Configure server-level CEA settings ##
## configureCEASettingsForServer("AppSrvCEANode01", "server1", "false | true", "30", "SIP_CTI_GATEWAY | THIRD_PARTY_WEB_SERVICE", "localhost", "5060", "TCP | UDP | TLS", "false | true", "ceauser", '""')

def configureCEASettingsForServer(nodeName, serverName, enableCEA, maxRequestHoldTime, telephonyAccessMethod, gatewayAddress, gatewayPort, gatewayProtocol, extractUsernameFromRequest, superUsername, thirdPartyWSDLProvider):
   deployment = AdminConfig.getid("/Deployment:commsvc/")
   deployedObject = AdminConfig.showAttribute(deployment, "deployedObject")
   rawTargetMappings = AdminConfig.showAttribute(deployedObject, "targetMappings")
   targetMappingList = convertToList(rawTargetMappings)
   for targetMapping in targetMappingList:
      currentTarget = AdminConfig.showAttribute(targetMapping, "target")
      if (currentTarget.find("ServerTarget") > 0):
         if ((AdminConfig.showAttribute(currentTarget, "nodeName") == nodeName) & (AdminConfig.showAttribute(currentTarget, "name") == serverName)):
            AdminConfig.modify(targetMapping, "[[enable " + enableCEA + "]]")
   if (len(AdminConfig.getid("/Node:" + nodeName + "/Server:" + serverName + "/CEASettings:/")) == 0):
      ceaSettingsID = AdminConfig.create("CEASettings", AdminConfig.getid("/Node:" + nodeName + "/Server:" + serverName + "/"), "[[telephonyAccessMethod " + telephonyAccessMethod + "]]")
      AdminConfig.create("CTIGateway", ceaSettingsID, "[[gatewayAddress " + gatewayAddress + "] [gatewayPort " + gatewayPort + "] [gatewayProtocol " + gatewayProtocol + "] [extractUsernameFromRequest " + extractUsernameFromRequest + "] [superUsername " + superUsername + "]]")
      AdminConfig.create("Commsvc", ceaSettingsID, "[[maxRequestHoldTime " + maxRequestHoldTime + "] [thirdPartyWSDLProvider " + thirdPartyWSDLProvider + "]]")
   else:
      AdminConfig.modify(AdminConfig.getid("/Node:" + nodeName + "/Server:" + serverName + "/CEASettings:/"), "[[telephonyAccessMethod " + telephonyAccessMethod + "]]")
      AdminConfig.modify(AdminConfig.getid("/Node:" + nodeName + "/Server:" + serverName + "/CEASettings:/CTIGateway:/"), "[[gatewayAddress " + gatewayAddress + "] [gatewayPort " + gatewayPort + "] [gatewayProtocol " + gatewayProtocol + "] [extractUsernameFromRequest " + extractUsernameFromRequest + "] [superUsername " + superUsername + "]]")
      AdminConfig.modify(AdminConfig.getid("/Node:" + nodeName + "/Server:" + serverName + "/CEASettings:/Commsvc:/"), "[[maxRequestHoldTime " + maxRequestHoldTime + "] [thirdPartyWSDLProvider " + thirdPartyWSDLProvider + "]]")
#endDef


## Configure cluster-level CEA settings ##
## configureCEASettingsForCluster("cluster1", "false | true", "30", "SIP_CTI_GATEWAY | THIRD_PARTY_WEB_SERVICE", "localhost", "5060", "TCP | UDP | TLS", "false | true", "ceauser", '""')

def configureCEASettingsForCluster(clusterName, enableCEA, maxRequestHoldTime, telephonyAccessMethod, gatewayAddress, gatewayPort, gatewayProtocol, extractUsernameFromRequest, superUsername, thirdPartyWSDLProvider):
   deployment = AdminConfig.getid("/Deployment:commsvc/")
   deployedObject = AdminConfig.showAttribute(deployment, "deployedObject")
   rawTargetMappings = AdminConfig.showAttribute(deployedObject, "targetMappings")
   targetMappingList = convertToList(rawTargetMappings)
   for targetMapping in targetMappingList:
      currentTarget = AdminConfig.showAttribute(targetMapping, "target")
      if (currentTarget.find("ClusteredTarget") > 0):
         if (AdminConfig.showAttribute(currentTarget, "name") == clusterName):
            AdminConfig.modify(targetMapping, "[[enable " + enableCEA + "]]")
   if (len(AdminConfig.getid("/ServerCluster:" + clusterName + "/CEASettings:/")) == 0):
      ceaSettingsID = AdminConfig.create("CEASettings", AdminConfig.getid("/ServerCluster:" + clusterName + "/"), "[[telephonyAccessMethod " + telephonyAccessMethod + "]]")
      AdminConfig.create("CTIGateway", ceaSettingsID, "[[gatewayAddress " + gatewayAddress + "] [gatewayPort " + gatewayPort + "] [gatewayProtocol " + gatewayProtocol + "] [extractUsernameFromRequest " + extractUsernameFromRequest + "] [superUsername " + superUsername + "]]")
      AdminConfig.create("Commsvc", ceaSettingsID, "[[maxRequestHoldTime " + maxRequestHoldTime + "] [thirdPartyWSDLProvider " + thirdPartyWSDLProvider + "]]")
   else:
      AdminConfig.modify(AdminConfig.getid("/ServerCluster:" + clusterName + "/CEASettings:/"), "[[telephonyAccessMethod " + telephonyAccessMethod + "]]")
      AdminConfig.modify(AdminConfig.getid("/ServerCluster:" + clusterName + "/CEASettings:/CTIGateway:/"), "[[gatewayAddress " + gatewayAddress + "] [gatewayPort " + gatewayPort + "] [gatewayProtocol " + gatewayProtocol + "] [extractUsernameFromRequest " + extractUsernameFromRequest + "] [superUsername " + superUsername + "]]")
      AdminConfig.modify(AdminConfig.getid("/ServerCluster:" + clusterName + "/CEASettings:/Commsvc:/"), "[[maxRequestHoldTime " + maxRequestHoldTime + "] [thirdPartyWSDLProvider " + thirdPartyWSDLProvider + "]]")
#endDef


## Show key attributes of all SIP Application Routers ##
## showAllRouters()

def showAllRouters():
   # Show key attributes of the default router
   showDefaultRouter()
   # Show key attributes of all custom routers
   showAllCustomRouters()
#endDef


## Show key attributes of the default SIP application router ##
## showDefaultRouter()

def showDefaultRouter():
   # Correct format of base SAR objects (if necessary)
   if (len(AdminConfig.list("SIPApplicationRouters")) == 0):
      AdminConfig.create("SIPApplicationRouters", AdminConfig.getid("/Cell:/"), [])
      AdminConfig.create("DefaultSIPApplicationRouter", AdminConfig.getid("/SIPApplicationRouters:/"), [])

   # Show key attributes of default router

   # Create list of clusters assigned to default router
   rawAllClusters = AdminConfig.getid("/ServerCluster:/")
   darClusterList = convertToList(rawAllClusters)
   rawClusterTargets = AdminConfig.getid("/SARClusterTarget:/")
   clusterTargetList = convertToList(rawClusterTargets)
   if (len(clusterTargetList) > 0):
      for clusterTarget in clusterTargetList:
         clusterName = AdminConfig.showAttribute(clusterTarget, "name")
         cluster = AdminConfig.getid("/ServerCluster:" + clusterName + "/")
         darClusterList.remove(cluster)

   # Create list of servers assigned to the default router
   darServerList = []

   # Add application servers that are not members of a cluster
   rawAllServers = AdminConfig.getid("/Server:/")
   allServerList = convertToList(rawAllServers)
   if (len(allServerList) > 0):
      for server in allServerList:
         if (AdminConfig.showAttribute(server, "serverType") == "APPLICATION_SERVER"):
            if (AdminConfig.showAttribute(server, "clusterName") is None):
               darServerList.append(server)

   # Remove application servers assigned to a custom router
   rawServerTargets = AdminConfig.getid("/SARServerTarget:/")
   serverTargetList = convertToList(rawServerTargets)
   if (len(serverTargetList) > 0):
      for serverTarget in serverTargetList:
         nodeName = AdminConfig.showAttribute(serverTarget, "nodeName")
         serverName = AdminConfig.showAttribute(serverTarget, "name")
         server = AdminConfig.getid("/Node:" + nodeName + "/Server:" + serverName + "/")
         darServerList.remove(server)

   print "\nRouter: DefaultSIPApplicationRouter"
   print "\nTargets and Application Startup Order:"
   if ((len(darClusterList) > 0) | (len(darServerList) > 0)):
      cellName = AdminConfig.showAttribute(AdminConfig.getid("/Cell:/"), "name")
      if (len(darClusterList) > 0):
         for cluster in darClusterList:
            clusterName = AdminConfig.showAttribute(cluster, "name")
            print "\n/ServerCluster:" + clusterName + "/"
            scope = "WebSphere:cell=" + cellName + ",cluster=" + clusterName
            apps = AdminApp.list(scope)
            apps = convertToList(apps)
            if (len(apps) > 0):
               for app in apps:
                  deployment = AdminConfig.getid("/Deployment:"+app+"/")
                  appDeployment = AdminConfig.showAttribute(deployment, "deployedObject")
                  weight = AdminConfig.showAttribute(appDeployment, "startingWeight")
                  print "   " + app + " (" + weight + ")"
            else:
               print "   No Applications Deployed"
      if (len(darServerList) > 0):
         for server in darServerList:
            serverName = server[0:server.find("(")]
            nodeName = server[server.find("nodes/")+6:server.find("servers/")-1]
            print "\n/Node:" + nodeName + "/Server:" + serverName + "/"
            scope = "WebSphere:cell=" + cellName + ",node=" + nodeName + ",server=" + serverName
            apps = AdminApp.list(scope)
            apps = convertToList(apps)
            if (len(apps) > 0):
               for app in apps:
                  deployment = AdminConfig.getid("/Deployment:"+app+"/")
                  appDeployment = AdminConfig.showAttribute(deployment, "deployedObject")
                  weight = AdminConfig.showAttribute(appDeployment, "startingWeight")
                  print "   " + app + " (" + weight + ")"
            else:
               print "   No Applications Deployed"

      print "\nUse the following command to modify the application startup order:"
      print "AdminApplication.configureStartingWeightForAnApplication(app, weight)"
   else:
      print "None"
#endDef


## Show key attributes of all custom SIP Application Routers ##
## showAllCustomRouters()

def showAllCustomRouters():
   # Correct format of base SAR objects (if necessary)
   if (len(AdminConfig.list("SIPApplicationRouters")) == 0):
      AdminConfig.create("SIPApplicationRouters", AdminConfig.getid("/Cell:/"), [])
      AdminConfig.create("DefaultSIPApplicationRouter", AdminConfig.getid("/SIPApplicationRouters:/"), [])

   # Show key attributes of all custom routers
   rawRouters = AdminConfig.getid("/SIPApplicationRouter:/")
   routerList = convertToList(rawRouters)
   if (len(routerList) > 0):
      for router in routerList:
         print "\nRouter: " + AdminConfig.showAttribute(router, "name")
         print "\nProvider: " + AdminConfig.showAttribute(router, "sipApplicationRouterProvider")
         print "\nTargets:"
         rawTargets = AdminConfig.showAttribute(router, "targets")
         targetList = convertToList(rawTargets)
         if (len(targetList) == 0):
            print "None"
         else:
            for target in targetList:
               if (target.find("ClusterTarget") > 0):
                  print "/ServerCluster:" + AdminConfig.showAttribute(target, "name") + "/"
            for target in targetList:
               if (target.find("ServerTarget") > 0):
                  print "/Node:" + AdminConfig.showAttribute(target, "nodeName") + "/Server:" + AdminConfig.showAttribute(target, "name") + "/"
   else:
      print "\nNo custom routers exist"
#endDef


## Show key attributes of a custom SIP application router ##
## showCustomRouter("RouterOne")

def showCustomRouter(routerName):
   # Correct format of base SAR objects (if necessary)
   if (len(AdminConfig.list("SIPApplicationRouters")) == 0):
      AdminConfig.create("SIPApplicationRouters", AdminConfig.getid("/Cell:/"), [])
      AdminConfig.create("DefaultSIPApplicationRouter", AdminConfig.getid("/SIPApplicationRouters:/"), [])

   # Show key attributes of custom router
   router = AdminConfig.getid("/SIPApplicationRouter:" + routerName + "/")
   print "\nRouter: " + AdminConfig.showAttribute(router, "name")
   print "\nProvider: " + AdminConfig.showAttribute(router, "sipApplicationRouterProvider")
   print "\nTargets:"
   rawTargets = AdminConfig.showAttribute(router, "targets")
   targetList = convertToList(rawTargets)
   if (len(targetList) == 0):
      print "None"
   else:
      for target in targetList:
         if (target.find("ClusterTarget") > 0):
            print "/ServerCluster:" + AdminConfig.showAttribute(target, "name") + "/"
      for target in targetList:
         if (target.find("ServerTarget") > 0):
            print "/Node:" + AdminConfig.showAttribute(target, "nodeName") + "/Server:" + AdminConfig.showAttribute(target, "name") + "/"
#endDef


## Create a new custom SIP Application Router ##
## createRouter("RouterOne", "RouterProviderOne")

def createRouter(routerName, routerProvider):
   # Correct format of base SAR objects (if necessary)
   if (len(AdminConfig.list("SIPApplicationRouters")) == 0):
      AdminConfig.create("SIPApplicationRouters", AdminConfig.getid("/Cell:/"), [])
      AdminConfig.create("DefaultSIPApplicationRouter", AdminConfig.getid("/SIPApplicationRouters:/"), [])

   # Create the router
   AdminConfig.create("SIPApplicationRouter", AdminConfig.getid("/SIPApplicationRouters:/"), "[[name " + routerName + "] [sipApplicationRouterProvider " + routerProvider + "]]")
#endDef


## Modify the name and provider of a SIP Application Router ##
## modifyRouter("RouterOne", "Router1", "RouterProvider1")

def modifyRouter(routerName, newRouterName, newRouterProvider):
   # Correct format of base SAR objects (if necessary)
   if (len(AdminConfig.list("SIPApplicationRouters")) == 0):
      AdminConfig.create("SIPApplicationRouters", AdminConfig.getid("/Cell:/"), [])
      AdminConfig.create("DefaultSIPApplicationRouter", AdminConfig.getid("/SIPApplicationRouters:/"), [])

   # Modify the name and provider of the router
   AdminConfig.modify(AdminConfig.getid("/SIPApplicationRouter:" + routerName + "/"), "[[name " + newRouterName + "] [sipApplicationRouterProvider " + newRouterProvider + "]]")
#endDef


## Delete a SIP Application Router and any targets assigned to it ##
## deleteRouter("RouterOne")

def deleteRouter(routerName):
   # Correct format of base SAR objects (if necessary)
   if (len(AdminConfig.list("SIPApplicationRouters")) == 0):
      AdminConfig.create("SIPApplicationRouters", AdminConfig.getid("/Cell:/"), [])
      AdminConfig.create("DefaultSIPApplicationRouter", AdminConfig.getid("/SIPApplicationRouters:/"), [])

   # Delete router and any targets assigned to it
   rawTargets = AdminConfig.showAttribute(AdminConfig.getid("/SIPApplicationRouter:" + routerName + "/"), "targets")
   targetList = convertToList(rawTargets)
   if (len(targetList) > 0):
     for target in targetList:
       AdminConfig.remove(target)
   AdminConfig.remove(AdminConfig.getid("/SIPApplicationRouter:" + routerName + "/"))
#endDef


## Move a server to a custom SIP Application Router ##
## moveServerToCustomRouter("AppSrvCEANode01", "server1", "RouterOne")

def moveServerToCustomRouter(nodeName, serverName, routerName):
   # Correct format of base SAR objects (if necessary)
   if (len(AdminConfig.list("SIPApplicationRouters")) == 0):
      AdminConfig.create("SIPApplicationRouters", AdminConfig.getid("/Cell:/"), [])
      AdminConfig.create("DefaultSIPApplicationRouter", AdminConfig.getid("/SIPApplicationRouters:/"), [])

   # Move server to default router
   rawTargets = AdminConfig.getid("/SARServerTarget:/")
   targetList = convertToList(rawTargets)
   if (len(targetList) > 0):
      for target in targetList:
         if (AdminConfig.showAttribute(target, "nodeName") == nodeName):
            if (AdminConfig.showAttribute(target, "name") == serverName):
               AdminConfig.remove(target)

   # Move server to custom router
   rawTargets = AdminConfig.showAttribute(AdminConfig.getid("/SIPApplicationRouter:" + routerName + "/"), "targets")
   targetList = convertToList(rawTargets)
   newTarget = AdminConfig.create("SARServerTarget", AdminConfig.getid("/SIPApplicationRouters:/"), "[[name " + serverName + "] [nodeName " + nodeName + "]]")
   targetList.append(newTarget)
   targetString = " ".join(targetList)
   AdminConfig.modify(AdminConfig.getid("/SIPApplicationRouter:" + routerName + "/"), "[[targets \"" + targetString + "\"]]")
#endDef


## Move a server to the default SIP Application Router ##
## moveServerToDefaultRouter("AppSrvCEANode01", "server1")

def moveServerToDefaultRouter(nodeName, serverName):
   # Correct format of base SAR objects (if necessary)
   if (len(AdminConfig.list("SIPApplicationRouters")) == 0):
      AdminConfig.create("SIPApplicationRouters", AdminConfig.getid("/Cell:/"), [])
      AdminConfig.create("DefaultSIPApplicationRouter", AdminConfig.getid("/SIPApplicationRouters:/"), [])

   # Move server to default router
   rawTargets = AdminConfig.getid("/SARServerTarget:/")
   targetList = convertToList(rawTargets)
   if (len(targetList) > 0):
      for target in targetList:
         if (AdminConfig.showAttribute(target, "nodeName") == nodeName):
            if (AdminConfig.showAttribute(target, "name") == serverName):
               AdminConfig.remove(target)
#endDef


## Move a cluster to a custom SIP Application Router ##
## moveClusterToCustomRouter("cluster1", "RouterOne")

def moveClusterToCustomRouter(clusterName, routerName):
   # Correct format of base SAR objects (if necessary)
   if (len(AdminConfig.list("SIPApplicationRouters")) == 0):
      AdminConfig.create("SIPApplicationRouters", AdminConfig.getid("/Cell:/"), [])
      AdminConfig.create("DefaultSIPApplicationRouter", AdminConfig.getid("/SIPApplicationRouters:/"), [])

   # Move cluster to default router
   rawTargets = AdminConfig.getid("/SARClusterTarget:/")
   targetList = convertToList(rawTargets)
   if (len(targetList) > 0):
      for target in targetList:
         if (AdminConfig.showAttribute(target, "name") == clusterName):
            AdminConfig.remove(target)

   # Move cluster to custom router
   rawTargets = AdminConfig.showAttribute(AdminConfig.getid("/SIPApplicationRouter:" + routerName + "/"), "targets")
   targetList = convertToList(rawTargets)
   newTarget = AdminConfig.create("SARClusterTarget", AdminConfig.getid("/SIPApplicationRouters:/"), "[[name " + clusterName + "]]")
   targetList.append(newTarget)
   targetString = " ".join(targetList)
   AdminConfig.modify(AdminConfig.getid("/SIPApplicationRouter:" + routerName + "/"), "[[targets \"" + targetString + "\"]]")
#endDef


## Move a cluster to the default SIP Application Router ##
## moveClusterToDefaultRouter("cluster1")

def moveClusterToDefaultRouter(clusterName):
   # Correct format of base SAR objects (if necessary)
   if (len(AdminConfig.list("SIPApplicationRouters")) == 0):
      AdminConfig.create("SIPApplicationRouters", AdminConfig.getid("/Cell:/"), [])
      AdminConfig.create("DefaultSIPApplicationRouter", AdminConfig.getid("/SIPApplicationRouters:/"), [])

   # Move cluster to default router
   rawTargets = AdminConfig.getid("/SARClusterTarget:/")
   targetList = convertToList(rawTargets)
   if (len(targetList) > 0):
      for target in targetList:
         if (AdminConfig.showAttribute(target, "name") == clusterName):
            AdminConfig.remove(target)
#endDef


## Start an application on a cluster ##
## startAppOnCluster("PlantsByWebSphereAjax", "Dmgr01Cell01", "cluster1")

def startAppOnCluster(appName, cellName, clusterName):
   rawMembers = AdminConfig.getid("/Cell:" + cellName + "/ServerCluster:" + clusterName + "/ClusterMember:/")
   members = convertToList(rawMembers)
   for member in members:
      serverName = AdminConfig.showAttribute(member, "memberName")
      nodeName = AdminConfig.showAttribute(member, "nodeName")
      rawAppManagers = AdminControl.queryNames("cell=" + cellName + ",node=" + nodeName + ",process=" + serverName + ",type=ApplicationManager,*")
      appManagers = convertToList(rawAppManagers)
      for appManager in appManagers:
         AdminControl.invoke(appManager, "startApplication", appName)
#endDef


## Save changes, synchronize, and sleep ##
## saveChanges()

def saveChanges():
   AdminConfig.save()
   if (AdminConfig.showAttribute(AdminConfig.getid("/Cell:" + cellName + "/"), "cellType") == "DISTRIBUTED"):
      print "\nSynchronizing changes"
      print time.ctime()
      AdminControl.invoke(AdminControl.queryNames("type=DeploymentManager,*"), "syncActiveNodes", "true")
      print "\nSleeping 60 seconds to allow all processing to complete"
      print time.ctime()
      time.sleep(60)
   elif (AdminConfig.showAttribute(AdminConfig.getid("/Cell:" + cellName + "/"), "cellRegistered") == "true"):
      print "\nSleeping 30 seconds to allow all processing to complete"
      print time.ctime()
      time.sleep(30)
#endDef


## Convert string list to Jython list ##
## convertToList(stringList)

def convertToList(inlist):
   outlist = []
   if (len(inlist) > 0):
      if (inlist[0] == '[' and inlist[len(inlist) - 1] == ']'):
         # Handle case where configuration name contains one or more spaces
         if (inlist[1] == "\"" and inlist[len(inlist)-2] == "\""):
            clist = inlist[1:len(inlist) -1].split(")\" ")
         else:
            clist = inlist[1:len(inlist) - 1].split(" ")
      else:
         clist = inlist.split(java.lang.System.getProperty("line.separator"))
      for elem in clist:
         elem = elem.rstrip();
         if (len(elem) > 0):
            if (elem[0] == "\"" and elem[len(elem) -1] != "\""):
               elem = elem+")\""
            outlist.append(elem)
   return outlist
#endDef


# Main - Server

if(len(sys.argv) == 6):
  pbxAppPath = sys.argv[0]
  plantsAppPath = sys.argv[1]
  wsAppPath = sys.argv[2]
  cellName = sys.argv[3]
  nodeName = sys.argv[4]
  serverName = sys.argv[5]
  pbxAddress = "localhost"
  pbxPort = "5060"
elif(len(sys.argv) == 8):
  pbxAppPath = sys.argv[0]
  plantsAppPath = sys.argv[1]
  wsAppPath = sys.argv[2]
  cellName = sys.argv[3]
  nodeName = sys.argv[4]
  serverName = sys.argv[5]
  pbxAddress = sys.argv[6]
  pbxPort = sys.argv[7]
else:
  print "\nUsage:"
  print "wsadmin -lang jython -f setupCEA.py <pbx_app_path> <plants_app_path> <ws_app_path> <cell_name> <node_name> <server_name>"
  print "-or-"
  print "wsadmin -lang jython -f setupCEA.py <pbx_app_path> <plants_app_path> <ws_app_path> <cell_name> <node_name> <server_name> <pbx_address> <pbx_port>"
  sys.exit()

print "\nConfiguring CEA Settings"
print time.ctime()
configureCEASettingsForServer(nodeName, serverName, 'true', '30', 'SIP_CTI_GATEWAY', pbxAddress, pbxPort, 'TCP', 'false', 'ceauser', '""')

print "\nDefining the JDBC provider for the CEA PlantsByWebSphereAjax sample application"
print time.ctime()
jdbcProvider = AdminTask.createJDBCProvider('[-scope Cell=' + cellName + ',Node=' + nodeName + ',Server=' + serverName + ' -databaseType Derby -providerType "Derby JDBC Provider" -implementationType "XA data source" -name "Derby Provider for PlantsByWebSphereAjax (XA)" -description "Derby embedded XA JDBC Provider. This provider is only configurable in version 6.0.2 and later nodes" -classpath [${DERBY_JDBC_DRIVER_PATH}/derby.jar ] -nativePath "" ]')

print "\nDefining the data source for the CEA PlantsByWebSphereAjax sample application"
print time.ctime()
AdminTask.createDatasource(jdbcProvider, '[-name PLANTSDB -jndiName jdbc/PlantsByWebSphereAjaxDataSource -dataStoreHelperClassName com.ibm.websphere.rsadapter.DerbyDataStoreHelper -containerManagedPersistence true -componentManagedAuthenticationAlias -xaRecoveryAuthAlias -configureResourceProperties [[databaseName java.lang.String "${APP_INSTALL_ROOT}/${CELL}/PlantsByWebSphereAjax.ear/Database/PLANTSDB"]]]')

print "\nInstalling the CEA PlantsByWebSphereAjax sample application"
print time.ctime()
AdminApp.install(plantsAppPath, '[ -nopreCompileJSPs -distributeApp -nouseMetaDataFromBinary -nodeployejb -appname PlantsByWebSphereAjax -createMBeansForResources -noreloadEnabled -nodeployws -validateinstall warn -noprocessEmbeddedConfig -filepermission .*\.dll=755#.*\.so=755#.*\.a=755#.*\.sl=755 -noallowDispatchRemoteInclude -noallowServiceRemoteInclude -asyncRequestDispatchType DISABLED -nouseAutoLink -MapModulesToServers [[ "PlantsByWebSphere EJB Module" PlantsByWebSphereAjax_EJB.jar,META-INF/ejb-jar.xml WebSphere:cell=' + cellName + ',node=' + nodeName + ',server=' + serverName + ' ][ "PlantsByWebSphere Web Application" PlantsByWebSphere.war,WEB-INF/web.xml WebSphere:cell=' + cellName + ',node=' + nodeName + ',server=' + serverName + ' ]]]' )

print "\nInstalling the sample IP-PBX application"
print time.ctime()
AdminApp.install(pbxAppPath, '[-nopreCompileJSPs -distributeApp -nouseMetaDataFromBinary -nodeployejb -appname commsvc.pbx -createMBeansForResources -noreloadEnabled -nodeployws -validateinstall warn -noprocessEmbeddedConfig -filepermission .*\.dll=755#.*\.so=755#.*\.a=755#.*\.sl=755 -noallowDispatchRemoteInclude -noallowServiceRemoteInclude -asyncRequestDispatchType DISABLED -nouseAutoLink -MapModulesToServers [[CSTASipPbx commsvc.pbx.war,WEB-INF/sip.xml WebSphere:cell=' + cellName + ',node=' + nodeName + ',server=' + serverName + ' ]]]')

print "\nInstalling the CEA Web services sample application"
print time.ctime()
AdminApp.install(wsAppPath, '[ -nopreCompileJSPs -distributeApp -nouseMetaDataFromBinary -nodeployejb -appname commsvc.ws.sample -createMBeansForResources -noreloadEnabled -nodeployws -validateinstall warn -noprocessEmbeddedConfig -filepermission .*\.dll=755#.*\.so=755#.*\.a=755#.*\.sl=755 -noallowDispatchRemoteInclude -noallowServiceRemoteInclude -asyncRequestDispatchType DISABLED -nouseAutoLink -MapModulesToServers [[ commsvc.ws.sample commsvc.ws.sample.war,WEB-INF/web.xml WebSphere:cell=' + cellName + ',node=' + nodeName + ',server=' + serverName + ' ]]]' )

print "\nSaving changes"
print time.ctime()
saveChanges()

print "\nStarting the Sample IP-PBX application"
print time.ctime()
appManager = AdminControl.queryNames('cell=' + cellName + ',node=' + nodeName + ',type=ApplicationManager,process=' + serverName + ',*')
AdminControl.invoke(appManager, 'startApplication', 'commsvc.pbx')

print "\nStarting the CEA PlantsByWebSphereAjax sample application"
print time.ctime()
AdminControl.invoke(appManager, 'startApplication', 'PlantsByWebSphereAjax')

print "\nStarting the CEA Web services sample application"
print time.ctime()
AdminControl.invoke(appManager, 'startApplication', 'commsvc.ws.sample')

# Main - Cluster

# if(len(sys.argv) == 5):
#   pbxAppPath = sys.argv[0]
#   plantsAppPath = sys.argv[1]
#   wsAppPath = sys.argv[2]
#   cellName = sys.argv[3]
#   clusterName = sys.argv[4]
#   pbxAddress = "localhost"
#   pbxPort = "5060"
# elif(len(sys.argv) == 7):
#   pbxAppPath = sys.argv[0]
#   plantsAppPath = sys.argv[1]
#   wsAppPath = sys.argv[2]
#   cellName = sys.argv[3]
#   clusterName = sys.argv[4]
#   pbxAddress = sys.argv[5]
#   pbxPort = sys.argv[6]
# else:
#   print "\nUsage:"
#   print "wsadmin -lang jython -f setupCEA.py <pbx_app_path> <plants_app_path> <ws_app_path> <cell_name> <cluster_name>"
#   print "-or-"
#   print "wsadmin -lang jython -f setupCEA.py <pbx_app_path> <plants_app_path> <ws_app_path> <cell_name> <cluster_name> <pbx_address> <pbx_port>"
#   sys.exit()

# print "\nConfiguring CEA Settings"
# print time.ctime()
# configureCEASettingsForCluster(clusterName, 'true', '30', 'SIP_CTI_GATEWAY', pbxAddress, pbxPort, 'TCP', 'false', 'ceauser', '""')

# print "\nDefining the JDBC provider for the CEA PlantsByWebSphereAjax sample application"
# print time.ctime()
# jdbcProvider = AdminTask.createJDBCProvider('[-scope Cell=' + cellName + ',Cluster=' + clusterName + ' -databaseType Derby -providerType "Derby JDBC Provider" -implementationType "XA data source" -name "Derby Provider for PlantsByWebSphereAjax (XA)" -description "Derby embedded XA JDBC Provider. This provider is only configurable in version 6.0.2 and later nodes" -classpath [${DERBY_JDBC_DRIVER_PATH}/derby.jar ] -nativePath "" ]')

# print "\nDefining the data source for the CEA PlantsByWebSphereAjax sample application"
# print time.ctime()
# AdminTask.createDatasource(jdbcProvider, '[-name PLANTSDB -jndiName jdbc/PlantsByWebSphereAjaxDataSource -dataStoreHelperClassName com.ibm.websphere.rsadapter.DerbyDataStoreHelper -containerManagedPersistence true -componentManagedAuthenticationAlias -xaRecoveryAuthAlias -configureResourceProperties [[databaseName java.lang.String "${APP_INSTALL_ROOT}/${CELL}/PlantsByWebSphereAjax.ear/Database/PLANTSDB"]]]')

# print "\nInstalling the CEA PlantsByWebSphereAjax sample application"
# print time.ctime()
# AdminApp.install(plantsAppPath, '[ -nopreCompileJSPs -distributeApp -nouseMetaDataFromBinary -nodeployejb -appname PlantsByWebSphereAjax -createMBeansForResources -noreloadEnabled -nodeployws -validateinstall warn -noprocessEmbeddedConfig -filepermission .*\.dll=755#.*\.so=755#.*\.a=755#.*\.sl=755 -noallowDispatchRemoteInclude -noallowServiceRemoteInclude -asyncRequestDispatchType DISABLED -nouseAutoLink -MapModulesToServers [[ "PlantsByWebSphere EJB Module" PlantsByWebSphereAjax_EJB.jar,META-INF/ejb-jar.xml WebSphere:cell=' + cellName + ',cluster=' + clusterName + ' ][ "PlantsByWebSphere Web Application" PlantsByWebSphere.war,WEB-INF/web.xml WebSphere:cell=' + cellName + ',cluster=' + clusterName + ' ]]]' )

# print "\nInstalling the sample IP-PBX application"
# print time.ctime()
# AdminApp.install(pbxAppPath, '[-nopreCompileJSPs -distributeApp -nouseMetaDataFromBinary -nodeployejb -appname commsvc.pbx -createMBeansForResources -noreloadEnabled -nodeployws -validateinstall warn -noprocessEmbeddedConfig -filepermission .*\.dll=755#.*\.so=755#.*\.a=755#.*\.sl=755 -noallowDispatchRemoteInclude -noallowServiceRemoteInclude -asyncRequestDispatchType DISABLED -nouseAutoLink -MapModulesToServers [[CSTASipPbx commsvc.pbx.war,WEB-INF/sip.xml WebSphere:cell=' + cellName + ',cluster=' + clusterName + ' ]]]')

# print "\nInstalling the CEA Web services sample application"
# print time.ctime()
# AdminApp.install(wsAppPath, '[ -nopreCompileJSPs -distributeApp -nouseMetaDataFromBinary -nodeployejb -appname commsvc.ws.sample -createMBeansForResources -noreloadEnabled -nodeployws -validateinstall warn -noprocessEmbeddedConfig -filepermission .*\.dll=755#.*\.so=755#.*\.a=755#.*\.sl=755 -noallowDispatchRemoteInclude -noallowServiceRemoteInclude -asyncRequestDispatchType DISABLED -nouseAutoLink -MapModulesToServers [[ commsvc.ws.sample commsvc.ws.sample.war,WEB-INF/web.xml WebSphere:cell=' + cellName + ',cluster=' + clusterName + ' ]]]' )

# print "\nSaving changes"
# print time.ctime()
# saveChanges()

# print "\nStarting the Sample IP-PBX application"
# print time.ctime()
# startAppOnCluster('commsvc.pbx', cellName, clusterName)

# print "\nStarting the CEA PlantsByWebSphereAjax sample application"
# print time.ctime()
# startAppOnCluster('PlantsByWebSphereAjax', cellName, clusterName)

# print "\nStarting the CEA Web services sample application"
# print time.ctime()
# startAppOnCluster('commsvc.ws.sample', cellName, clusterName)
